/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.layoutComponents;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.LinkedList;

import org.apache.jena.rdf.model.Resource;
import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.core.databinding.beans.PojoProperties;
import org.eclipse.swt.graphics.Color;
import org.eclipse.wb.swt.SWTResourceManager;

import eu.DECIDEh2020.architect.appManager.models.AppDescriptor;
import eu.DECIDEh2020.architect.appManager.models.Microservice;

import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.jface.text.TextViewer;

public class ProjectPageComposite extends Composite {
	private DataBindingContext m_bindingContext;

	private AppDescriptor project;
	private Text txtProjectname;
	private Color background;
	private Composite pp = this;
	private MicroserviceListComposite microserviceListComposite;
	private StyledText styledText;
	
	
	
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public ProjectPageComposite(Composite parent, AppDescriptor project) {
		super(parent, SWT.NONE);
		
		this.project = project;
		background= SWTResourceManager.getColor(SWT.COLOR_WHITE);
		setBackground(background);
		setContent();
		requestLayout();
	}

	private void setContent() {
		Control[] children = this.getChildren();
	    for (int i = 0 ; i < children.length; i++) {
	        children[i].dispose();
	    }
		setLayout(new GridLayout(1, true));
		
		CLabel lblOverview = new CLabel(this, SWT.NONE);
		lblOverview.setFont(SWTResourceManager.getFont("Segoe UI", 12, SWT.BOLD));
		lblOverview.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblOverview.setText("Overview");
		
		Label label = new Label(this, SWT.SEPARATOR | SWT.HORIZONTAL);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Composite container  = new Composite(this, SWT.NONE);
		container.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, true, true, 1, 1));
		container.setBackground(background);
		GridLayout gl_container = new GridLayout(2,true);
		gl_container.horizontalSpacing = 25;
		container.setLayout(gl_container);
		
		CLabel lblProject = new CLabel(container, SWT.NONE);
		lblProject.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
		lblProject.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblProject.setBottomMargin(0);
		lblProject.setTopMargin(0);
		lblProject.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblProject.setText("Project");
		
		CLabel lblMicroservices = new CLabel(container, SWT.NONE);
		lblMicroservices.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
		lblMicroservices.setTopMargin(0);
		lblMicroservices.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblMicroservices.setBottomMargin(0);
		lblMicroservices.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblMicroservices.setText("Microservices");
		

		
		Composite composite = new Composite(container, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1, 1));
		composite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite.setLayout(new GridLayout(1, false));
		
	
		
		
		
		Composite projectInfo = new Composite(composite, SWT.NONE);
		projectInfo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		projectInfo.setSize(202, 130);
		projectInfo.setBackground(background);
		projectInfo.setLayout(new GridLayout(2, false));
		
		Label lblProjectname = new Label(projectInfo, SWT.NONE);
		lblProjectname.setBackground(background);
		lblProjectname.setSize(67, 15);
		lblProjectname.setText("Projectname:");
		
		txtProjectname = new Text(projectInfo, SWT.BORDER);
		txtProjectname.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		txtProjectname.setSize(74, 21);
		
		Label lblProjectNfrs = new Label(projectInfo, SWT.NONE);
		lblProjectNfrs.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblProjectNfrs.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		lblProjectNfrs.setText("Project NFRs:");
		
		NfrListViewer nfrListViewer = new NfrListViewer(projectInfo, SWT.BORDER | SWT.V_SCROLL| SWT.MULTI);
		nfrListViewer.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		nfrListViewer.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
		
		CLabel lblDescription = new CLabel(projectInfo, SWT.NONE);
		lblDescription.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblDescription.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		lblDescription.setText("Description");
		
		TextViewer textViewer = new TextViewer(projectInfo, SWT.BORDER | SWT.WRAP);
		styledText = textViewer.getTextWidget();
		GridData gd_styledText = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_styledText.minimumHeight = 80;
		styledText.setLayoutData(gd_styledText);
		
		CLabel lblDeploymentOrder = new CLabel(composite, SWT.NONE);
		lblDeploymentOrder.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblDeploymentOrder.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
		lblDeploymentOrder.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblDeploymentOrder.setBottomMargin(0);
		lblDeploymentOrder.setTopMargin(0);
		lblDeploymentOrder.setText("Deployment Order");
		
		microserviceListComposite = new MicroserviceListComposite(composite, SWT.NONE, project, false,SWTResourceManager.getColor(SWT.COLOR_WHITE));
		microserviceListComposite.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true, 1, 1));
		
		
		
		
		
		txtProjectname.addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent e) {
				if (project != null) project.setName(txtProjectname.getText());
			}
		});
		
		
		
		ScrolledComposite scrolledComposite = new ScrolledComposite(container, SWT.V_SCROLL);
		
		
		scrolledComposite.addListener(SWT.Activate, new Listener() {
	        public void handleEvent(Event e) {
	            scrolledComposite.setFocus();
	        }
	    });
		
		  scrolledComposite.addListener(SWT.MouseVerticalWheel, new Listener() {
	            public void handleEvent(Event event) {
	                int wheelCount = event.count;
	                wheelCount = (int) Math.ceil(wheelCount / 3.0f);
	                while (wheelCount < 0) {
	                    scrolledComposite.getVerticalBar().setIncrement(4);
	                    wheelCount++;
	                }

	                while (wheelCount > 0) {
	                    scrolledComposite.getVerticalBar().setIncrement(-4);
	                    wheelCount--;
	                }
	            }
	        });
		  
		scrolledComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		
		
		
		Composite mcontainer = new Composite(scrolledComposite, SWT.NONE);
		mcontainer.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		mcontainer.setLayout(new GridLayout(1, false));
		addMicroservices(mcontainer);
		mcontainer.requestLayout();
		
		scrolledComposite.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, true, 1, 1));
		scrolledComposite.setContent(mcontainer);
		scrolledComposite.setExpandVertical( true );
		scrolledComposite.setExpandHorizontal( true );
		int width = scrolledComposite.getClientArea().width/2;
		scrolledComposite.setMinSize( mcontainer.computeSize( width, SWT.DEFAULT ) );
		
		scrolledComposite.addListener( SWT.Resize, event -> {
			  final int w = scrolledComposite.getClientArea().width/2;
			  scrolledComposite.setMinSize( mcontainer.computeSize( w, SWT.DEFAULT ) );
			} );
		
		//
		if(project != null && project.getProjectNfrs().size() != 0) {
			nfrListViewer.addSelection(new ArrayList<Resource>(project.getProjectNfrs()));
		}
		//when the selection in the nfr list is changed, change the underlying set of NFR enums
		nfrListViewer.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (project != null) {
					HashSet<Resource> mscNfrs = new HashSet<Resource>();
				for (Resource r: nfrListViewer.getSelection()) {
					try {
						mscNfrs.add(r);
					} catch (IllegalArgumentException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}
				project.setProjectNfrs(mscNfrs);
				mcontainer.requestLayout();
				}
			}
		});
		microserviceListComposite.addContentChangeListener(new ContentChangeListener() {

			@Override
			public void contentChanged() {
				refreshMicroservices(mcontainer);
				mcontainer.requestLayout();
				
				int width = scrolledComposite.getClientArea().width/2;
				scrolledComposite.setMinSize( mcontainer.computeSize( width, SWT.DEFAULT ) );
			}
			
		});
		
		m_bindingContext = initDataBindings();
		scrolledComposite.requestLayout();
		requestLayout();
		
	}
	
	public void refreshContent(AppDescriptor project) {
		this.project = project;
		setContent();
	}
	
	public void refreshMicroservices(Composite microserviceContainer) {
		
		Control[] children = microserviceContainer.getChildren();
	    for (int i = 0 ; i < children.length; i++) {
	        children[i].dispose();
	    }
	    addMicroservices(microserviceContainer);
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	
	
	private void addMicroserviceComposite(Composite container,Microservice microservice){
		MicroserviceComposite msvContainer = new MicroserviceComposite(container, microservice, background);
		msvContainer.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		msvContainer.addContentChangeListener(new ContentChangeListener() {

			@Override
			public void contentChanged() {
				microserviceListComposite.refresh();
				
			}
			
		});
	}
	
	
	private void addMicroservices(Composite container) {
		if (project== null)return;
		LinkedList<Microservice> microservices = project.getMicroservices();
		if(microservices.size()>0){
			for(Microservice m: microservices) {
				addMicroserviceComposite(container,m);
			}
			
		}
		//to test in designer
		
			/*Microservice m = new Microservice();
			addMicroserviceComposite(container,m);*/
		
		
	}
	protected DataBindingContext initDataBindings() {
		DataBindingContext bindingContext = new DataBindingContext();
		//
		IObservableValue observeTextTxtProjectnameObserveWidget = WidgetProperties.text(new int[]{SWT.Modify, SWT.FocusOut}).observe(txtProjectname);
		IObservableValue nameProjectObserveValue = PojoProperties.value("name").observe(project);
		bindingContext.bindValue(observeTextTxtProjectnameObserveWidget, nameProjectObserveValue, null, null);
		//
		IObservableValue observeTextStyledTextObserveWidget = WidgetProperties.text(SWT.Modify).observe(styledText);
		IObservableValue descriptionProjectObserveValue = PojoProperties.value("description").observe(project);
		bindingContext.bindValue(observeTextStyledTextObserveWidget, descriptionProjectObserveValue, null, null);
		//
		return bindingContext;
	}
}
