/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.appManager.models;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.UUID;

import org.apache.jena.rdf.model.Resource;

public class Microservice {
	
	private class InfrastructureRequirements{
		private String disk_min, disk_max, ram;

		/**
		 * @return the disk_min
		 */
		public String getDisk_min() {
			return disk_min;
		}

		/**
		 * @param disk_min the disk_min to set
		 */
		public void setDisk_min(String disk_min) {
			this.disk_min = disk_min;
		}

		/**
		 * @return the disk_max
		 */
		public String getDisk_max() {
			return disk_max;
		}

		/**
		 * @param disk_max the disk_max to set
		 */
		public void setDisk_max(String disk_max) {
			this.disk_max = disk_max;
		}

		/**
		 * @return the ram
		 */
		public String getRam() {
			return ram;
		}

		/**
		 * @param ram the ram to set
		 */
		public void setRam(String ram) {
			this.ram = ram;
		}
	}
	private String  ms_name, patterns[], programming_language ="", type, category, ms_repo ;
	private HashSet<Resource> nfrs;
	private UUID ms_uid;
	private boolean stateless = false, publicIP =false;
	private URI endpoint[], container_ref;
	private int deployment_order;
	private ArrayList<String> dependencies = new ArrayList<String>();
	private InfrastructureRequirements infrastructure_requirements; 
	
	/**
	 * @return the programmingLanguage
	 */
	public String getProgrammingLanguage() {
		return programming_language;
	}


	/**
	 * @param programmingLanguage the programmingLanguage to set
	 */
	public void setProgrammingLanguage(String programmingLanguage) {
		this.programming_language = programmingLanguage;
	}


	/**
	 * @return the category
	 */
	public String getCategory() {
		return category;
	}


	/**
	 * @param category the category to set
	 */
	public void setCategory(String category) {
		this.category = category;
	}


	/**
	 * @return the endpoint
	 */
	public URI[] getEndpoint() {
		return endpoint;
	}


	/**
	 * @param endpoint the endpoint to set
	 */
	public void setEndpoint(URI[] endpoint) {
		this.endpoint = endpoint;
	}


	/**
	 * @return the container_ref
	 */
	public URI getContainer_ref() {
		return container_ref;
	}


	/**
	 * @param container_ref the container_ref to set
	 */
	public void setContainer_ref(URI container_ref) {
		this.container_ref = container_ref;
	}


	/**
	 * @return the deployment_order
	 */
	public int getDeployment_order() {
		return deployment_order;
	}


	/**
	 * @param deployment_order the deployment_order to set
	 */
	public void setDeployment_order(int deployment_order) {
		this.deployment_order = deployment_order;
	}


	/**
	 * @return the infrastructure_requirements
	 */
	public InfrastructureRequirements getInfrastructure_requirements() {
		return infrastructure_requirements;
	}


	/**
	 * @param infrastructure_requirements the infrastructure_requirements to set
	 */
	public void setInfrastructure_requirements(InfrastructureRequirements infrastructure_requirements) {
		this.infrastructure_requirements = infrastructure_requirements;
	}


	/**
	 * @return the repo
	 */
	public String getRepo() {
		return ms_repo;
	}


	/**
	 * @param repo the repo to set
	 */
	public void setRepo(String repo) {
		this.ms_repo = repo;
	}


	/**
	 * @return the id
	 */
	public UUID getUID() {
		return ms_uid;
	}


	/**
	 * @param dependencies the dependencies to set
	 */
	public void setDependencies(ArrayList<String> dependencies) {
		this.dependencies = dependencies;
	}

	public void addDependency(String dependency) {
		this.dependencies.add(dependency);
	}



	
	public Microservice() {
		
		this("Microservice ", UUID.randomUUID());
		String uuid = this.getUIDstring().substring(0, 8);
		this.setName("Microservice " + uuid);
	}
	

	public Microservice(String name) {
		this(name, UUID.randomUUID());
	}
	
	public Microservice(UUID id) {
		this("Microservice", id);
	}
	
	public Microservice(HashSet<Resource> nfrs) {
		this("Microservice", nfrs);
	}
	
	public Microservice(String name,  HashSet<Resource> nfrs) {
		this(name, UUID.randomUUID(),nfrs );
	}
	
	public Microservice(String name,  UUID id) {
		this(name, id, new HashSet<Resource>() );
	}
	public Microservice(UUID id,  HashSet<Resource> nfrs) {
		this("Microservice", id,nfrs );
	}
	
	
	public Microservice(String name,  UUID id, HashSet<Resource> nfrs) {
		super();
		this.ms_name = name;
		this.ms_uid = id;
		this.stateless=true;
		this.nfrs = nfrs;
	}

	public String getUIDstring(){
		return ms_uid.toString();
	}
	
	public String getName(){
		return ms_name;
	}
	
	public void setName(String name) {
		this.ms_name = name;
	}

	public HashSet<Resource> getNfrs() {
		return nfrs;
	}

	public void setNfrs(HashSet<Resource> nfrs) {
		this.nfrs = nfrs;
	}

	public String[] getPatterns() {
		return patterns;
	}

	public void setPatterns(String[] patterns) {
		this.patterns = patterns;
	}

	public String getLanguage() {
		return programming_language;
	}

	public void setLanguage(String language) {
		this.programming_language = language;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public boolean isStateless() {
		return stateless;
	}
	
	public void setStateless(boolean stateless) {
		this.stateless = stateless;
	}


	/**
	 * @return the dependencies
	 */
	public ArrayList<String> getDependencies() {
		return dependencies;
	}




}
