/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.appManager.persistence;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import org.eclipse.jgit.api.AddCommand;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.PushCommand;
import org.eclipse.jgit.api.errors.CanceledException;
import org.eclipse.jgit.api.errors.ConcurrentRefUpdateException;
import org.eclipse.jgit.api.errors.DetachedHeadException;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.api.errors.InvalidConfigurationException;
import org.eclipse.jgit.api.errors.InvalidRemoteException;
import org.eclipse.jgit.api.errors.JGitInternalException;
import org.eclipse.jgit.api.errors.NoFilepatternException;
import org.eclipse.jgit.api.errors.NoHeadException;
import org.eclipse.jgit.api.errors.NoMessageException;
import org.eclipse.jgit.api.errors.RefNotFoundException;
import org.eclipse.jgit.api.errors.TransportException;
import org.eclipse.jgit.api.errors.WrongRepositoryStateException;
import org.eclipse.jgit.transport.CredentialsProvider;
import org.eclipse.jgit.transport.PushResult;
import org.eclipse.jgit.transport.UsernamePasswordCredentialsProvider;



public class GitHandler {
	public static class Repository {
		private String uri, path, host, user, password, name, port, protocol;
		private boolean secureStorage = true, newRepo = true, localRepo=true;

		public Repository(String uri, String path, String host, String user, String password, String port, String name, String protocol, boolean secureStorage) {
			this.uri = uri;
			this.path = path;
			this.host = host;
			this.user = user;
			this.name = name;
			this.port = port;
			this.protocol = protocol;
			this.password = password;
			this.setSecureStorage(secureStorage);
		}

		public Repository() {
			this.setSecureStorage(false);
		}

		/**
		 * @return the URI
		 */
		public String getURI() {
			return uri;
		}

		/**
		 * @param URI
		 *            the URI to set
		 */
		public void setURI(String uri) {
			this.uri = uri;
		}

		/**
		 * @return the path
		 */
		public String getPath() {
			return path;
		}

		/**
		 * @param path
		 *            the path to set
		 */
		public void setPath(String path) {
			this.path = path;
		}

		/**
		 * @return the host
		 */
		public String getHost() {
			return host;
		}

		/**
		 * @param host
		 *            the host to set
		 */
		public void setHost(String host) {
			this.host = host;
		}

		/**
		 * @return the user
		 */
		public String getUser() {
			return user;
		}

		/**
		 * @param user
		 *            the user to set
		 */
		public void setUser(String user) {
			this.user = user;
		}

		/**
		 * @return the password
		 */
		public String getPassword() {
			return password;
		}

		/**
		 * @param password
		 *            the password to set
		 */
		public void setPassword(String password) {
			this.password = password;
		}

		/**
		 * @return the secureStorage
		 */
		public boolean isSecureStorage() {
			return secureStorage;
		}

		/**
		 * @param secureStorage
		 *            the secureStorage to set
		 */
		public void setSecureStorage(boolean secureStorage) {
			this.secureStorage = secureStorage;
		}


		/**
		 * @return the name
		 */
		public String getName() {
			return name;
		}

		/**
		 * @param name the name to set
		 */
		public void setName(String name) {
			this.name = name;
		}

		/**
		 * @return the port
		 */
		public String getPort() {
			return port;
		}

		/**
		 * @param port the port to set
		 */
		public void setPort(String port) {
			this.port = port;
		}

		/**
		 * @return the protocol
		 */
		public String getProtocol() {
			return protocol;
		}

		/**
		 * @param protocol the protocol to set
		 */
		public void setProtocol(String protocol) {
			this.protocol = protocol;
		}

		/**
		 * @return the newRepo
		 */
		public boolean isNewRepo() {
			return newRepo;
		}

		/**
		 * @param newRepo the newRepo to set
		 */
		public void setNewRepo(boolean newRepo) {
			this.newRepo = newRepo;
		}

		/**
		 * @return the localRepo
		 */
		public boolean isLocalRepo() {
			return localRepo;
		}

		/**
		 * @param localRepo the localRepo to set
		 */
		public void setLocalRepo(boolean localRepo) {
			this.localRepo = localRepo;
		}

	}
	
	private Repository repository;
	private Git git;
	private File directory;
	private CredentialsProvider cp;
	
	public GitHandler(Repository repository) {
		this.repository=repository;
		if(repository.getName()!=null && !repository.getName().isEmpty() 
				&& repository.getPassword()!=null && !repository.getPassword().isEmpty()) {
			cp=new UsernamePasswordCredentialsProvider(repository.getName(), repository.getPassword());
		}
		
	}
	
	
	public void cloneRepositoryTo(File dir) throws InvalidRemoteException, TransportException, GitAPIException {
		
		
		Git git = Git.cloneRepository()
				.setURI(repository.getURI())
				.setCredentialsProvider(cp)
				.setDirectory(dir)
				.call();
		
		setDirectory(git.getRepository().getDirectory());
		git.close();
		
	}
	
	public void createRepositoryIn(File dir) throws IllegalStateException, GitAPIException {
		git = Git.init().setDirectory(dir).call();
		directory= git.getRepository().getDirectory();
		
	}
	
    public void cloneRepo() throws IOException, NoFilepatternException, GitAPIException {
        Git.cloneRepository()
                .setURI(repository.getURI())
                .setDirectory(directory)
                .call();
    }

    public void addToRepo() throws IOException, NoFilepatternException, GitAPIException {
        AddCommand add = git.add();
        add.addFilepattern(".").call();
    }

    public void commitToRepo(String message) throws IOException, NoHeadException,
            NoMessageException, ConcurrentRefUpdateException,
            JGitInternalException, WrongRepositoryStateException, GitAPIException {
        git.commit().setMessage(message).call();
    }

    public void pushToRepo() throws IOException, JGitInternalException,
            InvalidRemoteException, GitAPIException {
        PushCommand pc = git.push();
        pc.setCredentialsProvider(cp)
                .setForce(true)
                .setPushAll();
        try {
            Iterator<PushResult> it = pc.call().iterator();
            if (it.hasNext()) {
                System.out.println(it.next().toString());
            }
        } catch (InvalidRemoteException e) {
            e.printStackTrace();
        }
    }

    public void pullFromRepo() throws IOException, WrongRepositoryStateException,
            InvalidConfigurationException, DetachedHeadException,
            InvalidRemoteException, CanceledException, RefNotFoundException,
            NoHeadException, GitAPIException {
        git.pull().call();
    }

    public void setCredentials(String name,String password) {
    	cp=new UsernamePasswordCredentialsProvider(repository.getName(), repository.getPassword());
    }
    
    
	/**
	 * @return the repository
	 */
	public Repository getRepository() {
		return repository;
	}

	/**
	 * @param repository the repository to set
	 */
	public void setRepository(Repository repository) {
		this.repository = repository;
	}


	/**
	 * @return the directory
	 */
	public File getDirectory() {
		return directory;
	}


	/**
	 * @param directory the directory to set
	 */
	public void setDirectory(File directory) {
		this.directory = directory;
	}
}
